@@ -28,7 +28,7 @@
 #' @importFrom parsnip fit extract_parameter_set_dials
 #' @importFrom workflows workflow add_model add_recipe
 #' @importFrom tune tune_grid control_grid select_best finalize_workflow finalize_model tune_bayes control_grid control_bayes
-#' @importFrom yardstick metric_set accuracy kap roc_auc sens spec precision f_meas rmse rsq mae
+#' @importFrom yardstick metric_set accuracy kap roc_auc sens spec precision f_meas rmse rsq mae new_class_metric
 #' @importFrom rsample vfold_cv bootstraps validation_split
 #' @importFrom recipes all_nominal_predictors all_numeric_predictors all_outcomes all_predictors
 #' @importFrom finetune control_race tune_race_anova
@@ -55,15 +55,36 @@ train_models <- function(train_data,
   set.seed(seed)
 
   if (task == "classification") {
-    metrics <- metric_set(
-      accuracy,
-      kap,
-      sens,
-      spec,
-      precision,
-      f_meas,
-      roc_auc
-    )
+
+    if(is.null(summaryFunction)){
+      metrics <- metric_set(
+        accuracy,
+        kap,
+        sens,
+        spec,
+        precision,
+        f_meas,
+        roc_auc
+      )
+    }else{
+
+      newClassMetric <- new_class_metric(summaryFunction, "maximize")
+
+      assign(metric, newClassMetric)
+
+      metrics <- metric_set(
+        accuracy,
+        kap,
+        sens,
+        spec,
+        precision,
+        f_meas,
+        roc_auc,
+        !!sym(metric)
+      )
+
+
+    }
   } else {
     metrics <- metric_set(rmse, rsq, mae)
   }
@@ -113,6 +134,8 @@ train_models <- function(train_data,
     return(params_model)
   }
 
+  n_class <- length(levels(train_data[[label]]))
+
   for (algo in algorithms) {
     set.seed(seed)
     model <- NULL
@@ -125,14 +148,34 @@ train_models <- function(train_data,
 
     perform_tuning <- !is.null(algo_tune_params) && !is.null(resamples)
 
+
+    if (n_class > 2) {
+      if ("logistic_regression" %in% algorithms) {
+        logistic_regression = define_multinomial_regression_spec(task, tune = perform_tuning)
+      }
+
+      if ("penalized_logistic_regression" %in% algorithms) {
+        penalized_logistic_regression = define_penalized_multinomial_regression_spec(task, tune = perform_tuning)
+
+      }
+    } else {
+      if ("logistic_regression" %in% algorithms) {
+        logistic_regression =  define_logistic_regression_spec(task, tune = perform_tuning)
+      }
+
+      if ("penalized_logistic_regression" %in% algorithms) {
+        penalized_logistic_regression =  define_penalized_logistic_regression_spec(task, tune = perform_tuning)
+      }
+    }
+
     model_info <- switch(algo,
                          "random_forest" = define_random_forest_spec(task, train_data, label, tune = perform_tuning),
                          "ranger" = define_ranger_spec(task, train_data, label, tune = perform_tuning),
                          "c5.0" = define_c5_0_spec(task, tune = perform_tuning),
                          "xgboost" = define_xgboost_spec(task, train_data, label, tune = perform_tuning),
                          "lightgbm" = define_lightgbm_spec(task, train_data, label, tune = perform_tuning),
-                         "logistic_regression" = define_logistic_regression_spec(task, tune = perform_tuning),
-                         "penalized_logistic_regression" = define_penalized_logistic_regression_spec(task, tune = perform_tuning),
+                         "logistic_regression" = logistic_regression,
+                         "penalized_logistic_regression" = penalized_logistic_regression,
                          "decision_tree" = define_decision_tree_spec(task, tune = perform_tuning),
                          "svm_linear" = define_svm_linear_spec(task, tune = perform_tuning),
                          "svm_radial" = define_svm_radial_spec(task, tune = perform_tuning),
@@ -270,6 +313,7 @@ train_models <- function(train_data,
         e$message
       ))
     })
+
   }
 
   if (length(models) == 0) {
@@ -487,7 +531,6 @@ get_default_tune_params <- function(algo, train_data, label) {
          # 12. KNN
          "knn" = list(
            neighbors = c(3, 7),  # Narrowed range for efficiency
-           weight_func = c("rectangular", "triangular"),
            dist_power = c(1, 2)
          ),
 
@@ -558,4 +601,3 @@ get_default_tune_params <- function(algo, train_data, label) {
 
 
 
-
